////////////////////////////////////////////////////////////////////
// Node is a class that represents a single node in a coalescent
// tree. Coalescent trees are structured such that each node
// represents a branching point with one ancestor and two descendants,
// unless it is a terminal node, in which case it has no descendants.
public class Node {

    double time;   // The time between the node and its ancestor
    int visited;   // Placeholder for algorithms examining the tree
    int nodeId;    // An arbitrary identification number for bookkeeping
    int nMuts;     // Number of mutations between the node and its ancestor
    Node desc1;    // One of the node's descendants
    Node desc2;    // One of the node's descendants
    Node ancestor; // The node's ancestor node

    // Initialize an empty node
    public Node()
    {
	time = 0;
	visited = 0;
	nMuts = 0;
	desc1 = null;
	desc2 = null;
	ancestor = null;
	nodeId = 0;
    }

    // Initialize a node with a nodeID, which can be used for bookkeeping
    // before the node is assigned a position in a tree
    public Node(int index)
    {

	time  = 0;
	visited = 0;
	nMuts = 0;
	desc1    = null;
	desc2    = null;
	ancestor = null;
	nodeId   = index;
    }

    // Return the node as a human readable string
    public String toString()
    {
	String s = new String();;

	if (ancestor != null) {
	    s += ("Anc: " + ancestor.nodeId + ", ");
	} else {
	    s += new String(" Anc: 0, ");
	}

	if (desc1 != null) {
	    s += new String("Desc1: " + desc1.nodeId + ", ");
	} else {
	    s += new String("Desc1: x, ");
	}

	if (desc2 != null) {
	    s += new String("Desc2: "+ desc2.nodeId + ", ");
	} else {
	    s += new String("Desc2: x, ");
	}

	s += new String(" muts: " + nMuts + ", ");
	s += new String(" time: " + time);

	return s;
    }
}
